# -*- coding: utf-8 -*-
# Copyright 2023 Ubiratan S. Freitas


# This file is part of Lattesh.
# 
# Lattesh is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the 
# Free Software Foundation, either version 3 of the License, or (at your option) 
# any later version.
# 
# Lattesh is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
# more details.
# 
# You should have received a copy of the GNU General Public License along with 
# Lattice Tools. If not, see <https://www.gnu.org/licenses/>. 


from bpy.types import Panel


from .operators import have_scipy, have_occ


# Test functions for polling
def is_mode_object(context):
    return context.mode == 'OBJECT'

def is_active_object_mesh(context):
    return context.active_object is not None and context.active_object.type == 'MESH'
    








class View3DLatticePanel:
    bl_category = "Lattices"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'



class VIEW3D_PT_lattices_generate(View3DLatticePanel, Panel):
    bl_label = "Generate lattices"
    bl_idname = "VIEW3D_PT_lattices_generate"

    def draw(self, context):
        layout = self.layout

        lattice_mesh = context.scene.lattice_mesh

        layout.label(text="Regular cells")
        row = layout.row(align=True)
        row.label(text="Cell size")
        row.prop(lattice_mesh, "cellsize", text="")
        row = layout.row(align=True)
        row.label(text="Cell type")
        row.prop(lattice_mesh, "celltype", text="")
        row = layout.row(align=True)
        row.label(text="Number of cells")
        row.prop(lattice_mesh, "nelems", text="")
        layout.operator("object.create_meshlattice")
        layout.operator("object.fill_meshlattice")
        
        layout.label(text="Random lattice")
        row = layout.row(align=True)
        row.label(text="Characteristic length")
        row.prop(lattice_mesh, "charac_length", text="")
        layout.operator("object.fill_voronoilattice")


class VIEW3D_PT_lattice_OCC(View3DLatticePanel, Panel):
    bl_label = "Opencascade fill"
    bl_idname = "VIEW3D_PT_lattice_OCC"

    @classmethod
    def poll(cls, context):
        return have_occ and is_mode_object(context) and is_active_object_mesh(context)

    def draw(self, context):
        lattice_mesh = context.scene.lattice_mesh

        layout = self.layout
        row = layout.row(align=True)
        row.label(text="Strut radius")
        row.prop(lattice_mesh, "radius", text="")
        row = layout.row(align=True)
        row.label(text="Linear deflection")
        row.prop(lattice_mesh, "lin_deflect", text="")
        row = layout.row(align=True)
        row.label(text="Angular deflection")
        row.prop(lattice_mesh, "ang_deflect", text="")

        layout.operator("object.occlattice")


class VIEW3D_PT_lattice_metaball(View3DLatticePanel, Panel):
    bl_label = "Metaball fill"
    bl_idname = "VIEW3D_PT_lattice_metaball"
    bl_options = {"DEFAULT_CLOSED"}

    @classmethod
    def poll(cls, context):
        return is_mode_object(context) and is_active_object_mesh(context)

    def draw(self, context):
        lattice_mesh = context.scene.lattice_mesh

        layout = self.layout
        row = layout.row(align=True)
        row.label(text="Strut radius")
        row.prop(lattice_mesh, "radius", text="")
        row = layout.row(align=True)
        row.label(text="Radius ratio")
        row.prop(lattice_mesh, "r_ratio", text="")

        layout.operator("object.metalattice")


    