/**********************************************************************/
/** Copyright (C) 2021 - Université Catholique de Louvain		**/
/** affinistan version 2021.1						**/
/** Author: Mohamed HADDAD (immc-MEMA)				**/
/** Contact: mohamed.haddad@uclouvain.be				**/
/** Use of this source code is governed by an UCL-style license	**/ 
/** All Rights Reserved						**/
/** This project has received funding from the European 		**/
/** Union’s Horizon 2020 research and innovation program		**/
/** under grant agreement No 862015					**/
/***********************************************************************/

#ifndef BOX_AFFINE_H
#define BOX_AFFINE_H 1

#include "matprop.h"
#include "state.h"
 
/* Resolution of a constitutive model in elastoplasticity (EP), elasto-viscoplasticity (EVP) & viscoelasto-viscoplasticity (VE-VP) */

int constbox_evp(MATPROP *material, double dt, double *strstn, double ptn, double *dstrn, double *strs, double *p, double *pstrntn, double *pstrn);

int constbox_ep(MATPROP *material, double dt, double *strstn, double ptn, double *dstrn, double *strs, double *p, double *pstrntn, double *pstrn);

int constbox_vevp(MATPROP *material, STATE *tn, double dt, double *dstrn, double *strs, double *p, double *pstrn);


/* Resolution of the constitutive box in EP, EVP & VE-VP with an affine formulation */

int box_affine (double *dstrn, STATE *tn, double *strst0, double dt, double tau, MATPROP *inclprop, double *s, \
                 double ***C_tau_s, double **eps_tau_s, STATE *tr, double *Cmstrs);

#endif
