/**********************************************************************/
/** Copyright (C) 2021 - Université Catholique de Louvain		**/
/** secantmfh version 2021.1						**/
/** Author: Mohamed HADDAD (immc-MEMA)				**/
/** Contact: mohamed.haddad@uclouvain.be				**/
/** Use of this source code is governed by an UCL-style license	**/ 
/** All Rights Reserved						**/
/** This project has received funding from the European 		**/
/** Union’s Horizon 2020 research and innovation program		**/
/** under grant agreement No 862015					**/
/***********************************************************************/

#ifndef STATE_H
#define STATE_H

typedef struct State {
  double *strn;  		/** Strain tensor 								**/
  double *strs;  		/** Stress tensor 								**/
  double p;      		/** Accumulated plasticity - scalar 						**/
  double *pstrn;  		/** Plastic strain 								**/
  double **Si;        		/** Deviatoric part of viscous stress relative to Gi (matrix: nbrGi*6)  	**/
  double **Sigma_Hj;  		/** Volumetric part of viscous stress relative to kj (matrix: nbrKj*6) 	**/
  double strs_eq2nd;    	/** Second moment estimation of strs 						**/
  double dstrs_unload_eq2nd;   /** Second moment estimation of dstrs_unload(tn) = strs(tn) - strs_res(tn) : will be used just for residual states **/
  
} STATE;

typedef struct Statemacro {
  double *strn;        	/** Macroscopic strain tensor 						**/
  double *strs;        	/** Macroscopic stress tensor 						**/
  double **Cmacro;		/** Macroscopic elastic tensor 						**/
  double *Beta_macro;  	/** Macroscopic eigenstress tensor 						**/
} STATEMACRO;


#endif
